package example_person_animals_ver2;

import java.util.ArrayList;

public class Person {
	private String name;
	private ArrayList<Animal> pets = new ArrayList<>();
	private ArrayList<Flyer> flyers = new ArrayList<>();

	public Person(String name) {
		this.name = name;
	}
	
	public boolean addPet(Animal a) {
		if(!pets.contains(a)) {
			pets.add(a);
			if(a instanceof Flyer) {
				Flyer f = (Flyer)a;
				flyers.add(f);
			}
			return true;
		}
		return false;
	}
	
	public String birdsFlyAndSoar() {
		String msg = "";
		for(Flyer f : flyers) {
			msg += f.fly() + ", " + f.soar() + "\n";
		}
		return msg;
	}

	public ArrayList<Dog> getDogs() {
		ArrayList<Dog> dogs = new ArrayList<>();
		for(Animal a : pets) {
			if(a instanceof Dog) {
				dogs.add((Dog)a);
			}
		}
		return dogs;
	}

	public String getName() {
		return name;
	}

	public int getNumPets() {
		return pets.size();
	}

	public int getNumFlyers() {
		return flyers.size();
	}

	public Animal getPet(int i) {
		if( i>=0 && i<pets.size()) {
			return pets.get(i);
		}
		return null;
	}
	
	public Animal getPet(String name) {
		Animal a = new Dog(name);
		// or...
		// Animal a = new Bird(name);
		if(pets.contains(a)) {
			int loc = pets.indexOf(a);
			return pets.get(loc);
		}
		return null;
	}
	
	public String makeSounds() {
		String msg = "";
		for(Animal a : pets) {
			msg += a.makeSound() + " ";
		}
		return msg;
	}
	
	public ArrayList<Flyer> removeFlyers3() {
		ArrayList<Flyer> remFlyers = new ArrayList<>();
		// Loop over flyers & remove from pets
		for(Flyer f : flyers) {
			Animal a = (Animal)f;
			if(pets.contains(a)) {// Unnecessary, if addPet works correctly, it will be in there
				pets.remove(a);
				remFlyers.add(f);
			}
		}
		flyers.clear();
		return remFlyers;
	}

	public ArrayList<Flyer> removeFlyers() {
		ArrayList<Flyer> remFlyers = new ArrayList<>();
		for(Flyer f : flyers) {// Loop over flyers & remove from pets
			Animal a = (Animal)f;
			pets.remove(a);
			remFlyers.add(f);
		}
		flyers.clear(); // Remove all flyers
		return remFlyers;
	}

// Doesn't work.
//	public ArrayList<Flyer> removeFlyers() {
//		ArrayList<Flyer> remFlyers = flyers;
//		for(Flyer f : flyers) {// Loop over flyers & remove from pets
//			Animal a = (Animal)f;
//			pets.remove(a);
//		}
//		flyers.clear(); // Remove all flyers
//		return remFlyers;
//	}

	public Animal removePet2(int i) {
		if(i<0 || i>=pets.size()) {
			return null;
		}
		Animal a = pets.get(i);
		pets.remove(i);
		if(a instanceof Flyer) {
			Flyer f = (Flyer)a;
			flyers.remove(f);
		}
		return a;
	}

	public Animal removePet(int i) {
		if(i<0 || i>=pets.size()) {
			return null;
		}
		Animal a = pets.get(i);
		if(a instanceof Flyer) {
			Flyer f = (Flyer)a;
			flyers.remove(f);
		}
		return pets.remove(i);
	}
	
	public Animal removePet(String name) {
		Animal a = new Dog(name);
		int loc = pets.indexOf(a);
		return removePet(loc);
	}
	
	@Override
	public String toString() {
		String msg = name + "'s pets:\n";
		for(Animal a : pets) {
			msg += a + "\n";
		}
		msg += "\nFlyers\n";
		for(Flyer f : flyers) {
			msg += f + "\n";
		}
		return msg;
	}
}
