package example_person_animals_ver2;

import java.util.ArrayList;

public class PersonTest {

	public static void main(String[] args) {
		testAddPet_Success();
		testAddPet_Fail();
		testBirdsFlyAndSoar_FlyersOnly();
		testBirdsFlyAndSoar_MixAnimals();
		testToString();		
		testGetPet_Success();
		testGetPet_Fail();
		testGetPetWithName_Success();
		testGetPetWithName_Fail();		
		testMakeSounds();
		testRemoveFlyers();
		testRemovePet_Success();
		testRemovePet_Fail();
		testRemovePetWithName_Success();
		testRemovePetWithName_Fail();
		testGetDogs();
	}
	
	private static void testAddPet_Success() {
		System.out.println("testAddPet_Success()");
		
		Person p = new Person("Ulanda");
		System.out.println("Num pets before adds=" + p.getNumPets());

		Dog d = new Dog("Arlo");
		Bird b = new Bird("Tweety");
		boolean isAdded = p.addPet(d);
		System.out.println("Is: " + d + " added? " + isAdded);
		isAdded = p.addPet(b);
		System.out.println("Is: " + b + " added? " + isAdded);
		System.out.println("Num pets after 2 adds=" + p.getNumPets());
	}

	private static void testAddPet_Fail() {
		System.out.println("\ntestAddPet_Fail()");
		
		Person p = new Person("Ulanda");
		System.out.println("Num pets before adds=" + p.getNumPets());

		Dog d = new Dog("Arlo");
		Bird b = new Bird("Arlo");
		boolean isAdded = p.addPet(d);
		System.out.println("Is: " + d + " added? " + isAdded);
		isAdded = p.addPet(b);
		System.out.println("Is: " + b + " added? " + isAdded);
		System.out.println("Num pets after 2 adds=" + p.getNumPets());
	}
	
	private static void testBirdsFlyAndSoar_FlyersOnly() {
		System.out.println("\n-->testBirdsFlyAndSoar_FlyersOnly()");
		Person p = new Person("Tachala");
		Bird b1 = new Bird("Tweety");
		Bird b2 = new Bird("Blu");
		Bird b3 = new Bird("Zazu");
		p.addPet(b1);
		p.addPet(b2);
		p.addPet(b3);
		
		System.out.println(p.getNumFlyers() + " birds should fly and soar: \n");
		System.out.println(p.birdsFlyAndSoar());
	}
	
	private static void testBirdsFlyAndSoar_MixAnimals() {
		System.out.println("\n-->testBirdsFlyAndSoar_MixAnimals()");
		Person p = new Person("Tachala");
		Bird b1 = new Bird("Tweety");
		Dog d1 = new Dog("Bongo");
		Bird b2 = new Bird("Donald");
		Dog d2 = new Dog("Pluto");
		
		p.addPet(b1);
		p.addPet(d1);
		p.addPet(b2);
		p.addPet(d2);
		
		System.out.println(p.getNumFlyers() + " birds should fly and soar: \n");
		System.out.println(p.birdsFlyAndSoar());
	}

	private static void testToString() {
		System.out.println("\ntestToString()");
		
		Person p = new Person("Ulanda");
		Dog d = new Dog("Arlo");
		Bird b = new Bird("Tweety");
		p.addPet(d);
		p.addPet(b);
		System.out.println(p);

	}

	private static void testGetPet_Success() {
		System.out.println("\ntestGetPet_Success()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Expecting 3 pets:");
		for(int i=0; i<p.getNumPets(); i++) {
			System.out.println((i+1) + ". " + p.getPet(i));
		}
	}

	private static void testGetPet_Fail() {
		System.out.println("\ntestGetPet_Fail()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Person has 3 pets, trying to get one at index=5");
		System.out.println(p.getPet(5));

		System.out.println("Person has 3 pets, trying to get one at index=-3");
		System.out.println(p.getPet(-3));
	}

	private static void testGetPetWithName_Success() {
		System.out.println("\ntestGetPetWithName_Success()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Person has 3 pets, trying to get \"Gigi\"");
		System.out.println(p.getPet("Gigi"));
	}

	private static void testGetPetWithName_Fail() {
		System.out.println("\ntestGetPetWithName_Fail()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Person has 3 pets, trying to get \"Snoopy\"");
		System.out.println(p.getPet("Snoopy"));
	}

	private static void testMakeSounds() {
		System.out.println("\ntestMakeSounds()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Person has 3 pets, p.makeSounds():");
		System.out.println(p.makeSounds());
	}

	private static void testRemoveFlyers() {
		System.out.println("\ntestRemoveFlyers()");
		
		Person p = new Person("Kurt");
		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));
		p.addPet(new Bird("Woodstock"));

		ArrayList<Flyer> removedFlyers = p.removeFlyers();

		// Expected pets after remove
		String msgExpPets = String.format("Expected: pets after remove:%s, %s", "Arlo", "Gigi");
		System.out.println(msgExpPets);

		// Actual pets after remove. Build actual pets list.
		String msgActPets = "";
		String msgPetList = "";
		for(int i=0; i<p.getNumPets(); i++) {
			msgPetList += p.getPet(i).getName() + ", ";
		}
		msgActPets = String.format("Actual:   pets after remove:%s", msgPetList);
		System.out.println(msgActPets);

		// Expected flyers after remove
		String msgExpFlyers = String.format("Expected:    removed flyers:%s, %s", "Tweety", "Woodstock");
		System.out.println(msgExpFlyers);
		
		// Actual flyers after remove. Build removed flyers list.
		String msgActFlyers = "";
		String msgFlyerList = "";
		for(Flyer f : removedFlyers) {
			msgFlyerList += ((Animal)f).getName() + ", ";
		}
		msgActFlyers = String.format("Actual:      removed flyers:%s", msgFlyerList);
		System.out.println(msgActFlyers);
	}

	private static void testRemovePet_Success() {
		System.out.println("\ntestRemovePet_Success()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Num pets before p.remove(1)=" + p.getNumPets());

		Animal a = p.removePet(1);
		
		System.out.println("Num pets after p.remove(1)=" + p.getNumPets());
		System.out.println("Pet removed=" + a);
	}

	private static void testRemovePet_Fail() {
		System.out.println("\ntestRemovePet_Fail()");
		
		Person p = new Person("Ulanda");
		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Num pets before p.remove(...)=" + p.getNumPets());
		Animal a = p.removePet(5);
		System.out.println("Num pets after p.remove(5)=" + p.getNumPets() + ", Pet removed=" + a);
		
		a = p.removePet(-2);
		System.out.println("Num pets after p.remove(-2)=" + p.getNumPets() + ", Pet removed=" + a);
	}

	private static void testRemovePetWithName_Success() {
		System.out.println("\ntestRemovePetWithName_Success()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Num pets before p.remove(\"Tweety\")=" + p.getNumPets());

		Animal a = p.removePet(1);
		
		System.out.println("Num pets after p.remove(\"Tweety\")=" + p.getNumPets());
		System.out.println("Pet removed=" + a);
	}

	private static void testRemovePetWithName_Fail() {
		System.out.println("\ntestRemovePetWithName_Fail()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Num pets before p.remove(\"Snoopy\")==" + p.getNumPets());

		Animal a = p.removePet(5);
		
		System.out.println("Num pets after p.remove(\"Snoopy\")=" + p.getNumPets());
		System.out.println("Pet removed=" + a);
		
	}

	private static void testGetDogs() {
		System.out.println("\ntestGetDogs()");
		
		Person p = new Person("Ulanda");

		p.addPet(new Dog("Arlo"));
		p.addPet(new Bird("Tweety"));
		p.addPet(new Dog("Gigi"));

		System.out.println("Num pets before p.remove(\"Snoopy\")==" + p.getNumPets());

		ArrayList<Dog> dogs = p.getDogs();
		
		System.out.println("Should be two dogs:");
		
		for(Dog d : dogs) {
			System.out.println(d);
		}
	}

}
