package example_person_clone_deep;

import java.util.ArrayList;
import java.util.Arrays;

// Deep copy
public class Person implements Cloneable {

	private String name;
	private ArrayList<Integer> scores = new ArrayList<>();
	
	public Person(String name) {
		this.name = name;
	}
	
	public String getName() {
		return name;
	}

	public int getScore( int i ) { 
		return scores.get(i); 
	}
	
	public void addScore( int score ) { 
		scores.add(score); 
	}

	public Object clone() throws CloneNotSupportedException {
		Person clone = (Person)super.clone();
		// Create a new list of scores by cloning the original
		ArrayList<Integer> scoresClone = (ArrayList<Integer>)scores.clone();
		// Then set the clone's list to be the cloned list
		clone.scores = scoresClone;
		return clone;
	}
	
	public String toString() {
		String msg = ", Scores: ";
		for( int score : scores )
			msg += score + ", ";
		return "Name: " + name + msg;
	}
	
	public static void main(String[] args) throws CloneNotSupportedException {
		
		ArrayList<Integer> scores = new ArrayList<>( 
				Arrays.asList(85, 72, 96));

		Person p = new Person("Alex");
		p.addScore(89);
		p.addScore(93);
		Person clone = (Person)p.clone();
		clone.scores.set(0, 45);
		clone.scores.set(1, 55);
		clone.addScore(33);
		System.out.println("Original");
		System.out.println(p);
		System.out.println("Clone");
		System.out.println(clone);
	}
		
}
