package exercise_drawing_shapes;

import java.util.List;

public class DrawingTest {

	public static void main(String[] args) {
		testCreation_And_AddShape();
		testAddMultipleShapes();
		testGetShape_ValidLoc();
		testGetShape_InvalidLoc();
		testRemoveShape_ValidLoc();
		testRemoveShape_InvalidLoc();
		testGetBigShapes_SmallShapes();
		testGetBigShapes_MixShapes();
		testGetRectangleDimensions();
		testGetRectangleDimensions_NoRectangles();
		
	}
	
	private static void testCreation_And_AddShape() {
		System.out.println("-->testCreation_And_AddShape()");
		Drawing d = new Drawing();
		Shape c = new Circle("Red", 3.2);
		
		//getting shape in index 0
		System.out.println("Shape in index 0 before addShape(c) = " + d.getShape(0));
		
		d.addShape(c);
		System.out.println("Shape in index 0 after addShape(c) = " + d.getShape(0));
	}
	
	private static void testAddMultipleShapes() {
		System.out.println("\n-->testAddMultipleShapes()");
		Drawing d = new Drawing();
		Shape c = new Circle("Red", 3.2);
		Shape r = new Rectangle("Green", 5.0, 3.6);
		
		//getting shape in indeces 0 and 1
		System.out.println("Shape in index 0 before addShape() = " + d.getShape(0));
		System.out.println("Shape in index 1 before addShape() = " + d.getShape(1));
		
		d.addShape(c);
		d.addShape(r);
		System.out.println("Shape in index 0 after addShape() = " + d.getShape(0));
		System.out.println("Shape in index 1 after addShape() = " + d.getShape(1));
	}
	
	private static void testGetShape_ValidLoc() {
		System.out.println("\n-->testGetShape_ValidLoc()");
		Drawing d = getMixShapes();
		int loc = 3;
		
		Shape retrieved = d.getShape(loc);
		System.out.println("Shape retrieved at index " + loc + "= " + retrieved);
	}
	
	private static void testGetShape_InvalidLoc() {
		System.out.println("\n-->testGetShape_InvalidLoc()");
		Drawing d = getMixShapes();
		int loc = 7;
		
		Shape retrieved = d.getShape(loc);
		System.out.println("Shape retrieved at index " + loc + "= " + retrieved);
	}
	
	private static void testRemoveShape_ValidLoc() {
		System.out.println("\n-->testRemoveShape_ValidLoc()");
		Drawing d = getCircles();
		int loc = 3;
		
		Shape removed = d.removeShape(loc);
		System.out.println("Shape removed at index " + loc + "= " + removed);
	}
	
	private static void testRemoveShape_InvalidLoc() {
		System.out.println("\n-->testRemoveShape_InvalidLoc()");
		Drawing d = getCircles();
		int loc = 7;
		
		Shape removed = d.removeShape(loc);
		System.out.println("Shape removed at index " + loc + "= " + removed);
	}
	
	private static void testGetBigShapes_SmallShapes() {
		System.out.println("\n-->testGetBigShapes_SmallShapes()");
		Drawing d = getCircles();
		
		double bigArea = 50.0;
		List<Shape> retrieved = d.getBigShapes(bigArea);
		
		System.out.println("Shapes with an area bigger than " + bigArea);
		if(retrieved.size() > 0) {
			for(Shape s : retrieved) {
				System.out.println(s);
			}
		} else {
			System.out.println("No big shapes.");
		}
	}
	
	private static void testGetBigShapes_MixShapes() {
		System.out.println("\n-->testGetBigShapes_MixShapes()");
		Drawing d = getMixShapes();
		
		double bigArea = 50.0;
		List<Shape> retrieved = d.getBigShapes(bigArea);
		
		System.out.println("Shapes with an area bigger than " + bigArea);
		if(retrieved.size() > 0) {
			for(Shape s : retrieved) {
				System.out.println(s);
			}
		} else {
			System.out.println("No big shapes.");
		}
	}
	
	private static void testGetRectangleDimensions() {
		System.out.println("\n-->testGetRectangleDimensions()");
		Drawing d = getRectangles();
		System.out.println(d.getRectangleDimensions());
	}
	
	private static void testGetRectangleDimensions_NoRectangles() {
		System.out.println("\n-->testGetRectangleDimensions_NoRectangles()");
		Drawing d = getCircles();
		String rectDimensions = d.getRectangleDimensions();
		if(!rectDimensions.isEmpty()) {
			System.out.println(d.getRectangleDimensions());
		} else {
			System.out.println("No rectangles.");
		}
		
	}
	
	
	/**
	 * Helpers
	 */
	private static Drawing getCircles() {
		Drawing d = new Drawing();
		Circle c1 = new Circle("Red", 3.0);
		Circle c2 = new Circle("Blue", 2.5);
		Circle c3 = new Circle("Yellow", 1.0);
		Circle c4 = new Circle("Green", 1.5);
		Circle c5 = new Circle("Orange", 2.3);
		
		d.addShape(c1);
		d.addShape(c2);
		d.addShape(c3);
		d.addShape(c4);
		d.addShape(c5);
		
		return d;
	}
	
	private static Drawing getRectangles() {
		Drawing d = new Drawing();
		Rectangle r1 = new Rectangle("Red", 5.0, 7.0);
		Rectangle r2 = new Rectangle("Blue", 10.0, 7.3);
		Rectangle r3 = new Rectangle("Yellow", 3.7, 9.7);
		Rectangle r4 = new Rectangle("Green", 2.0, 3.0);
		Rectangle r5 = new Rectangle("Orange", 5.0, 7.0);
		
		d.addShape(r1);
		d.addShape(r2);
		d.addShape(r3);
		d.addShape(r4);
		d.addShape(r5);
		
		return d;
	}
	
	private static Drawing getMixShapes() {
		Drawing d = new Drawing();
		Rectangle r1 = new Rectangle("Red", 5.0, 7.0);
		Rectangle r2 = new Rectangle("Blue", 10.0, 7.3);
		Rectangle r3 = new Rectangle("Yellow", 3.7, 9.7);
		Circle c1 = new Circle("Red", 3.0);
		Circle c2 = new Circle("Blue", 7.5);
		
		d.addShape(r1);
		d.addShape(r2);
		d.addShape(r3);
		d.addShape(c1);
		d.addShape(c2);
		
		return d;
	}
}
